/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: input.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>

#include "vac/vacum.h"

/* Input (Un)Registration */

static struct vac_input_ops *inputs;

int vac_register_input(struct vac_input_ops *ops)
{
	ops->next = inputs;
	inputs    = ops;

	return 0;
}

int vac_unregister_input(struct vac_input_ops *ops)
{
	struct vac_input_ops *io = inputs, *prev = NULL;
	while (io) {
		if (io == ops) {
			if (prev)
				prev->next = io->next;
			else
				inputs = NULL;
		}
		prev = io;
		io   = io->next;
	}
	return 0;
}

static struct vac_input_ops *__find_format(char *format)
{
	struct vac_input_ops *ops = inputs;
	while (ops) {
		if (!strcmp(ops->format, format))
			return ops;
		ops = ops->next;
	}
	return NULL;
}

static int __match_ext(char *name, char *extensions)
{
	char ex[32], *pe;
	char *ext, *p;
	
	ext = strrchr(name, '.');
	if (ext) {
		ext++;
		p = extensions;
		for(;;) {
			pe = ex;
			while (*p != '\0' && *p != ',')
				*pe++ = *p++;
			*pe = '\0';
			if (!strcasecmp(ex, ext))
				return 1;
			if (*p == '\0')
				break;
			p++;
		}
	}
	return 0;
}

static struct vac_input_ops *__find_extension(char *location)
{
	struct vac_input_ops *ops = inputs;
	while (ops) {
		if (__match_ext(location, ops->ext))
			return ops;
		ops = ops->next;
	}
	return NULL;
}

static void __input_free(struct vac_input *in)
{
	free(in->location);
	free(in);
}

int vac_input_open(char *location, struct vac_input_param *ip, struct vac_input **inp)
{
	struct vac_input_ops *ops = NULL;
	int err;

	if (ip->format)
		ops = __find_format(ip->format);
	else
		ops = __find_extension(location);

	if (ops) {
		struct vac_input *in = malloc(sizeof(*in));
		if (!in)
			return -1;
		memset(in, 0, sizeof(*in));

		in->ops = ops;

		in->location = strdup(location);
		in->width  = ip->width;
		in->height = ip->height;
		in->actual_frate = in->target_frate = ip->frate;

		in->diff[0] = ip->diff[0];
		in->diff[1] = ip->diff[1];

		err = ops->open(in);
		if (err) {
			__input_free(in);
			return err;
		}

		*inp = in;
		return 0;
	} else 
		return VAC_ERR_FORMAT;
}

int vac_input_close(struct vac_input *vi)
{
	int err = vi->ops->close(vi);
	if (vi->frame)
		vac_frame_put(vi->frame);
	__input_free(vi);
	return err;
}

int vac_input_read(struct vac_input *vi, struct vac_frame **frame)
{	
	struct vac_frame *f;
	int err = vi->ops->read(vi, &f);
	if (!err) {
		if (vi->frame) {
			vac_frame_put(vi->frame);
			vi->frameno++;
		}

		vi->frame = vac_frame_get(f);
		*frame = f;
		vac_debug(9, "frameno %d", vi->frameno);
	}
	return err;
}

int vac_input_seek(struct vac_input *vi, int frameno)
{
	int err = vi->ops->seek(vi, frameno);
	if (!err) {
		if (vi->frame) {
			/* Invalidate cached frame */
			vac_frame_put(vi->frame);
			vi->frame = NULL;
		}
		vi->frameno = frameno;
		vac_debug(9, "frameno %d", vi->frameno);
	}

	return err;
}

/* Initialization */

void __attribute__ ((constructor)) vac_input_init(void)
{
	extern void __vac_input_init();
	__vac_input_init();
}
