/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: yuv.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>
#include <math.h>

#include <sys/stat.h>

#include <vac/vacum.h>

struct yuv_priv {
	int   fd;
	float mse;
	int   si;
	int   flen;
};

static int yuv_read(struct vac_input *vi, struct vac_frame **f)
{
	struct yuv_priv *priv = vi->priv;
	struct vac_frame *frame;
	int r;

	frame = vac_frame_alloc(priv->flen);
	if (!frame)
		return -1;
	vac_frame_fill(frame, VAC_PIX_YUV420P, vi->width, vi->height);

	r = read(priv->fd, frame->data, priv->flen);
	if (r != priv->flen) {
		vac_frame_put(frame);
		*f = NULL;
		if (r >= 0)
			r = VAC_ERR_EOF;
	} else {
		*f = frame;
		r = 0;
	}

	return r;
}

static int yuv_seek(struct vac_input *vi, int frameno)
{
	struct yuv_priv *priv = vi->priv;
	if (lseek(priv->fd, frameno * priv->flen, SEEK_SET) < 0)
		return -1;
	return 0;
}

static inline int __compare_frames(struct vac_input *vi, struct vac_frame_size *fs, int si)
{
	struct yuv_priv *priv = vi->priv;
	char *buf, *frame[2];
	int ylen, flen;
	float mse;

	ylen = fs->width * fs->height;
	flen = ylen + ylen / 2;

	buf = malloc(flen * 2);
	if (!buf)
		return 0;

	frame[0] = buf; frame[1] = buf + flen;

	if (read(priv->fd, buf, flen * 2) < 0)
		goto out;
	lseek(priv->fd, 0, SEEK_SET);

	mse = vac_mse(frame[0] + ylen, frame[1] + ylen, ylen / 2);
	if (mse < priv->mse) {
		priv->mse  = mse;
		priv->si   = si;
		priv->flen = flen;

		vi->width  = fs->width;
		vi->height = fs->height;
	}

	vac_debug(9, "YUV: %s MSE(Y) %f MSE (UV): %f", fs->name,
		vac_mse(frame[0], frame[1], ylen), mse);
out:
	free(buf);
	return 0;
}

static int __detect_frame_size(struct vac_input *vi, size_t file_size)
{
	struct yuv_priv *priv = vi->priv;
	struct vac_frame_size *fs = vac_fsize_table;
	int si = 0, flen;

	while (fs->name) {
		vac_frame_get_length(VAC_PIX_YUV420P, fs->width, fs->height, &flen);	
		if (!(file_size % flen))
			__compare_frames(vi, fs, si);
		fs++; si++;
	}

	if (priv->si < 0)
		return VAC_ERR_FSIZE;
	return 0;
}

static int yuv_open(struct vac_input *vi)
{
	struct yuv_priv *priv;
	struct stat st;
	int err = -1;

	priv = malloc(sizeof(*priv));
	if (!priv)
		return -1;
	vi->priv = priv;

	priv->fd = open(vi->location, O_RDONLY);
	if (priv->fd < 0)
		goto failed;

	if (fstat(priv->fd, &st) < 0)
		goto failed;

	if (!vi->width) {
		priv->mse = 100;
		priv->si  = -1;

		err = __detect_frame_size(vi, st.st_size);
		if (err)
			goto failed;
	} else
		vac_frame_get_length(VAC_PIX_YUV420P, vi->width, vi->height, &priv->flen);

	vi->total_frames = st.st_size / priv->flen;
	
	return 0;

failed:
	if (priv->fd >= 0)
		close(priv->fd);
	free(priv);
	vi->priv = NULL;
	return err;
}

static int yuv_close(struct vac_input *vi)
{
	struct yuv_priv *priv = vi->priv;
	close(priv->fd);
	return 0;
}



static struct vac_input_ops yuv_input_ops = {
	.format = "yuv",
	.ext    = "yuv",
	.desc   = "Raw YUV format",
	.open   = yuv_open,
	.close  = yuv_close,
	.seek   = yuv_seek,
	.read   = yuv_read,
};

void yuv_init(void)
{
	vac_register_input(&yuv_input_ops);
}

void yuv_deinit(void)
{
	vac_unregister_input(&yuv_input_ops);
}
