/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: pixel.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <stdint.h>

#include <vac/frame.h>
#include <vac/pixel.h>
#include <vac/errno.h>

/* 
 * Pixel conversion and frame initialization routines are based on
 *	FFMPEG Misc image conversion routines
 *	Copyright (c) 2001, 2002, 2003 Fabrice Bellard.
 */

struct vac_pix_format vac_pixfmt_table[] = {
	/* YUV formats */
	[VAC_PIX_YUV420P] = {
		.name = "yuv420p",
		.channels = 3,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PLANAR,
		.depth = 8,
		.x_chroma_shift = 1,.y_chroma_shift = 1,
	},
	[VAC_PIX_YUV422P] = {
		.name = "yuv422p",
		.channels = 3,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PLANAR,
		.depth = 8,
		.x_chroma_shift = 1,.y_chroma_shift = 0,
	},
	[VAC_PIX_YUV444P] = {
		.name = "yuv444p",
		.channels = 3,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PLANAR,
		.depth = 8,
		.x_chroma_shift = 0,.y_chroma_shift = 0,
	},
	[VAC_PIX_YUV422] = {
		.name = "yuv422",
		.channels = 1,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 8,
		.x_chroma_shift = 1,.y_chroma_shift = 0,
	},
	[VAC_PIX_YUV410P] = {
		.name = "yuv410p",
		.channels = 3,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PLANAR,
		.depth = 8,
		.x_chroma_shift = 2,.y_chroma_shift = 2,
	},
	[VAC_PIX_YUV411P] = {
		.name = "yuv411p",
		.channels = 3,
		.color_type = VAC_COLOR_YUV,
		.pixel_type = VAC_PIX_PLANAR,
		.depth = 8,
		.x_chroma_shift = 2,.y_chroma_shift = 0,
	},

	/* RGB formats */
	[VAC_PIX_RGB24] = {
		.name = "rgb24",
		.channels = 3,
		.color_type = VAC_COLOR_RGB,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 8,
	},
	[VAC_PIX_BGR24] = {
		.name = "bgr24",
		.channels = 3,
		.color_type = VAC_COLOR_RGB,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 8,
	},
	[VAC_PIX_RGBA32] = {
		.name = "rgba32",
		.channels = 4,.is_alpha = 1,
		.color_type = VAC_COLOR_RGB,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 8,
	},
	[VAC_PIX_RGB565] = {
		.name = "rgb565",
		.channels = 3,
		.color_type = VAC_COLOR_RGB,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 5,
	},
	[VAC_PIX_RGB555] = {
		.name = "rgb555",
		.channels = 4,.is_alpha = 1,
		.color_type = VAC_COLOR_RGB,
		.pixel_type = VAC_PIX_PACKED,
		.depth = 5,
	},
};

int vac_frame_fill(struct vac_frame *f, int pix, int width, int height)
{
	struct vac_pix_format *pf = &vac_pixfmt_table[pix];
	int p0_len, p1_len, vline, hline;

	f->width  = width;
	f->height = height;

	p0_len = width * height;
	switch (pix) {
	case VAC_PIX_YUV420P:
	case VAC_PIX_YUV422P:
	case VAC_PIX_YUV444P:
	case VAC_PIX_YUV410P:
	case VAC_PIX_YUV411P:
		hline = (width  + (1 << pf->x_chroma_shift) - 1) >> pf->x_chroma_shift;
		vline = (height + (1 << pf->y_chroma_shift) - 1) >> pf->y_chroma_shift;
		p1_len = vline * hline;
		f->plane[0] = f->data;
		f->plane[1] = f->plane[0] + p0_len;
		f->plane[2] = f->plane[1] + p1_len;
		f->hline[0] = width; f->vline[0] = height;
		f->hline[1] = hline; f->vline[1] = vline;
		f->hline[2] = hline; f->vline[2] = vline;
		f->length = p0_len + 2 * p1_len;
		break;

	case VAC_PIX_RGB24:
	case VAC_PIX_BGR24:
		f->plane[0] = f->data;
		f->plane[1] = NULL;
		f->plane[2] = NULL;
		f->hline[0] = width * 3; f->vline[0] = height;
		f->length = p0_len * 3;
		break;

	case VAC_PIX_RGBA32:
		f->plane[0] = f->data;
		f->plane[1] = NULL;
		f->plane[2] = NULL;
		f->hline[0] = width * 4; f->vline[0] = height;
		f->length = p0_len * 4;
		break;

	case VAC_PIX_RGB555:
	case VAC_PIX_RGB565:
	case VAC_PIX_YUV422:
		f->plane[0] = f->data;
		f->plane[1] = NULL;
		f->plane[2] = NULL;
		f->hline[0] = width * 2; f->vline[0] = height;
		f->length = p0_len * 2;
		break;

	default:
		f->plane[0] = NULL;
		f->plane[1] = NULL;
		f->plane[2] = NULL;
		f->plane[3] = NULL;
		return VAC_ERR_PIXFMT;
	}
	return 0;
}

int vac_frame_get_length(int pix, int width, int height, int *length)
{
	struct vac_frame dummy;
	int err = vac_frame_fill(&dummy, pix, width, height);
	if (!err) 
		*length = dummy.length;
	return err;
}
