/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: display.c,v 1.1.1.1 2003/08/13 06:11:01 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <getopt.h>
#include <time.h>

#include <gtk/gtk.h>

#include <vac/vacum.h>

#include "vout.h"
#include "display.h"

static struct vac_input_group *ins;

static int ui_enabled = 1;
static int playing;

void display_destroy(void)
{
	int i;
	for (i=0; i < ins->num; i++) {
		struct vout *v = ins->input[i]->out;
		if (v)
			vout_close(v);
	}
	vout_destroy();
}

int display_create(struct vac_input_group *inputs, int row)
{
	int width, height;
	int w, h, aw, ah, maxw, maxh;
	int x, y, i;

	ins = inputs;
	
	width  = ins->input[0]->width;
	height = ins->input[0]->height;

	/* Actual size including intraframe spacing */
	aw = width + 4; ah = height + 2;

	maxw = aw * row;
	maxh = ah * 3;

	/* Calculate size of the video window */
	w = aw * ins->num;
	if (w > maxw) {
		h = (w / maxw + (w % maxw ? 1:0)) * ah;
		w = maxw;
	} else
		h = ah;

	if (ui_enabled) {
		if (ui_init(50, 400, ins, row) < 0)
			return -1;
	}

	if (vout_create(w, h) < 0)
		return -1;

	x = y = 0;
	for (i=0; i < ins->num; i++) {
		struct vout_param p = {x, y, width, height};
		struct vout *v;
		int err = vout_open(&p, &v);
		if (err) {
			vac_perror(err, "Video output initialization failed");
			display_destroy();
			return err;
		}
		ins->input[i]->out = v;

		x += aw;
		if (x >= maxw) {
			y += ah;
			x  = 0;
		}
	}

	display_frame(NULL);
	return 0;
}

int display_frame(void *scroll)
{	
	int i, err, eof = 0;
	for (i=0; i < ins->num; i++) {
		struct vout *v = ins->input[i]->out;
		struct vac_frame *f;

		err = vac_input_read(ins->input[i], &f);
		if (err) {
			eof = 1;
			if (err != VAC_ERR_EOF) {
				vac_perror(err, "Read from %s failed",
						ins->input[i]->location);
				break;
			} else
				continue;
		}
		vout_display(v, f);
		vac_frame_put(f);

		if (ui_enabled)
			ui_update_info(ins->input[i]);
	}

 	if (ui_enabled && !scroll)
		ui_set_scroll(ins->input[0]->frameno);

	if (eof)
		playing = 0;

	return 0;
}

void display_scroll(int pos)
{
	int i;

	vac_debug(9, "Scroll pos %d", pos);

	for (i = 0; i < ins->num; i++)
		vac_input_seek(ins->input[i], pos);

	if (!playing)
		display_frame((void *)1);
}

#define MIN_DELAY  (10000000)
#define NANOSEC    (1000000000)

/* Measure minimum delay in nanoseconds */
static long __get_min_delay(clockid_t cid)
{
#if 0
	struct timespe ts1, ts2;
	struct timespec s = { 0, 0 };
	long avg = 0, elapsed, i;

	for (i = 0; i < 5; i++) {
		clock_gettime(cid, &ts1);
		nanosleep(&s, NULL);
		clock_gettime(cid, &ts2);

		elapsed = ts2.tv_nsec - ts1.tv_nsec;
		if (elapsed < 0)
			elapsed += NANOSEC;

		vac_debug(8, "elapsed %ld", elapsed);

		avg += elapsed;
	}
	return (avg / 5);
#else
	return MIN_DELAY;
#endif
}

void display_playback(void)
{
	struct timespec ts1, ts2;
	clockid_t cid;
	long min_delay;

	// Interval between frames (nanoseconds)
	long between = NANOSEC / ins->input[0]->target_frate;

	if (playing)
		return;

#if 0
	if (clock_getcpuclockid (0, &cid) < 0) {
		vac_info("CPU clock is not available. Using real-time clock.");
		cid = CLOCK_REALTIME;
	}
#else
	cid = CLOCK_REALTIME;
#endif

	min_delay = __get_min_delay(cid);

	if (ui_enabled)
		ui_set_status("Playing");
	playing = 1;

	while (playing) {
		long elapsed, delay;

		// Start time
		clock_gettime(cid, &ts1);

		display_frame(NULL);

		// Handle GTK events
		if (ui_enabled) {
			int q = FALSE, i;
			for (i=0; i<5 && !q && gtk_events_pending(); i++)
				q = gtk_main_iteration();
			if (q)
				break;
		}

		// End time
		clock_gettime(cid, &ts2);

		elapsed = ts2.tv_nsec - ts1.tv_nsec;
		if (elapsed < 0)
			elapsed += NANOSEC;

		delay = between - elapsed;

		vac_debug(8, "between %ld elapsed %ld delay %ld (min_delay %ld)",
				between, elapsed, delay, min_delay);

		if (delay < 0) {
			// Don't sleep
			continue;
		} else if (delay < min_delay)
			delay = 0;
		else
			delay -= min_delay;

		{
			struct timespec s = { 0, delay };
			nanosleep(&s, NULL);
		}
	}

	if (ui_enabled)
		ui_set_status("Ready");
	playing = 0;
}

void display_stop(void)
{
	playing = 0;
}

void display_loop(void)
{
	signal(SIGTERM, SIG_DFL);
	signal(SIGINT,  SIG_DFL);

	if (ui_enabled)
		ui_loop();
	else
		display_playback();
}

void display_disable_ui(void)
{
	ui_enabled = 0;
}
