/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: main.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <getopt.h>

#include <vac/vacum.h>

#include "display.h"

static struct vac_input_group inputs;

static struct vac_input_param iparam = {
	.frate = 15.0,
};

static struct option main_lopts[] = {
	{ "help",     0, 0, 'h' },
	{ "size",     1, 0, 's' },
	{ "fps",      1, 0, 'r' },
	{ "row",      1, 0, 'l' },
	{ "diff",     0, 0, 'd' },
	{ "debug",    1, 0, 'D' },
	{ "noui",     0, 0, 'U' },
	{ "format",   1, 0, 'f' },
	{ 0, 0, 0, 0 }
};

static char main_sopts[] = "-hd:s:l:MYD:df:U";

static char main_help[] = 
	"VACUM Play version " VERSION " \n"
	"Usage:\n"
	"\tvacplay [options]\n"
	"Options:\n"
	"\t--format -f <format>      File format\n"
	"\t--size -s <WxH>           Frame size\n"
	"\t--fps -r <FPS>            Frame rate (default is 15)\n\n"

	"\t--diff -d                 Visual diff\n"

	"\t--row -l <N>              Display N frames per row (default is 4)\n"

	"\t--debug -D <level>        Set debug level (0 - 9)\n"
	"\t--noui -U                 Disable UI\n"

	"\nExamples:\n"
	"\tvacplay --size qcif orig.yuv err.yuv --diff\n";

int main(int argc, char **argv)
{
	int opt = 0;
	int err, row;

	row = 3;

	while ((opt=getopt_long(argc, argv, main_sopts, main_lopts, NULL)) != -1) {
		switch(opt) {
		case 'D':
			vac_debug_level = atoi(optarg);
			break;

		case 'U':
			display_disable_ui();
			break;

		case 's':
			vac_get_fsize(optarg, &iparam.width, &iparam.height);
			break;

		case 'r':
			iparam.frate = atof(optarg);
			break;

		case 'f':
			if (iparam.format)
				free(iparam.format);
			iparam.format = strdup(optarg);
			break;

		case 'l':
			row = atoi(optarg);
			break;

		case 'd':
			if (inputs.num < 2) {
				vac_error("Diff has to be preceded by at least 2 inputs");
				exit(1);
			}

			iparam.format  = "diff";
			iparam.diff[0] = inputs.input[inputs.num - 2];
			iparam.diff[1] = inputs.input[inputs.num - 1];

			optarg = "diff";

		case 1:
			err = vac_input_add(&inputs, optarg, &iparam);
			if (err) {
				vac_perror(err, "Failed to add input %s", optarg);
				exit(1);
			}

			/* Reset per input parameters */
			iparam.diff[0] = NULL; iparam.diff[1] = NULL;
			iparam.format  = NULL;

			break;

		case 'h':
		default:
			printf(main_help);
			exit(0);
		}
	}

	argc -= optind;
	argv += optind;

	if (inputs.num < 1) {
		printf(main_help);
		exit(1);
	}

	vac_input_validate(&inputs);

	if (inputs.err_fsize) {
		vac_error("Frame size mismatch. Side-by-side display is not possible.");
		exit(1);
	}

	if (inputs.err_frate)
		vac_info("Frame rate mismatch. Results may be inaccurate.");

	if (inputs.err_total)
		vac_info("Total number of frames mismatch. Results may be inaccurate.");

	if (display_create(&inputs, row))
		exit(1);

	display_loop();

	display_destroy();

	return 0;
}
