/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: vacpsnr.c,v 1.1.1.1 2003/08/13 06:11:01 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <getopt.h>

#include "vac/vacum.h"

static struct vac_input_group inputs;

static float avg_psnr[3];
static int   avg_only;

void do_psnr(int frame, struct vac_frame *f1, struct vac_frame *f2)
{
	float p[3];
	int i;

	if (f1) {
		for (i=0; i < 3; i++) {
			p[i] = vac_psnr(f1->plane[i], f2->plane[i], 
						f1->hline[i] * f1->vline[i]);
			avg_psnr[i] += p[i];
		}
		if (!avg_only)
			printf("Frame: %.4d  PSNR(Y): %2.2f  PSNR(U): %2.2f  PSNR(V): %2.2f\n",
				frame, p[0], p[1], p[2]);
	} else {
		for (i=0; i < 3; i++)
			avg_psnr[i] /= frame;

                printf("Average:  PSNR(Y): %2.2f  PSNR(U): %2.2f  PSNR(V): %2.2f\n",
                        avg_psnr[0], avg_psnr[1], avg_psnr[2]);
	}
}

void process_inputs(void)
{
	struct vac_frame *f1, *f2;
	int err;

	while (1) {
		err = vac_input_read(inputs.input[0], &f1);
		if (err) {
			if (err == VAC_ERR_EOF)
				break;
			vac_perror(err, "Read from %s failed", inputs.input[0]->location);
			return;
		}

		err = vac_input_read(inputs.input[1], &f2);
		if (err) {
			vac_frame_put(f1);
			if (err == VAC_ERR_EOF)
				break;
			vac_perror(err, "Read from %s failed", inputs.input[1]->location);
			return;
		}
	
		do_psnr(inputs.input[0]->frameno, f1, f2);

		vac_frame_put(f1);
		vac_frame_put(f2);
	}
	do_psnr(inputs.input[0]->frameno + 1, NULL, NULL);
}

static struct option main_lopts[] = {
	{ "help",     0, 0, 'h' },
	{ "size",     1, 0, 's' },
	{ "fps",      1, 0, 'r' },
	{ "average",  0, 0, 'A' },
	{ "debug",    1, 0, 'D' },
	{ "format",   1, 0, 'f' },
	{ 0, 0, 0, 0 }
};

static char main_sopts[] = "-hs:r:dD:f:A";

static char main_help[] = 
	"VACUM PSNR version " VERSION " \n"
	"Usage:\n"
	"\tvacpsnr [options] file1 file2\n"
	"Options:\n"
	"\t--format -f <format>      File format\n"
	"\t--size -s <WxH>           Frame size\n"
	"\t--fps -r <FPS>            Frame rate (default is 15)\n"
	"\t--debug -D <level>        Set debug level (0 - 9)\n"
	"\t--average -A              Display average PSNR only\n"

	"\nExamples:\n"
	"\tvacpsnr orig.yuv err.yuv\n";

int main(int argc, char **argv)
{
	struct vac_input_param vip;
	int opt = 0;
	int err;

	memset(&vip, 0, sizeof(vip));
	
	vip.frate = 15;

	while ((opt=getopt_long(argc, argv, main_sopts, main_lopts, NULL)) != -1) {
		switch(opt) {
		case 'D':
			vac_debug_level = atoi(optarg);
			break;

		case 's':
			vac_get_fsize(optarg, &vip.width, &vip.height);
			break;

		case 'r':
			vip.frate = atof(optarg);
			break;

		case 'f':
			if (vip.format)
				free(vip.format);
			vip.format = strdup(optarg);
			break;

		case 'A':
			avg_only = 1;
			break;

		case 1:
			err = vac_input_add(&inputs, optarg, &vip);
			if (err) {
				vac_perror(err, "Failed to add input %s", optarg);
				exit(1);
			}
			break;

		case 'h':
		default:
			printf(main_help);
			exit(0);
		}
	}

	argc -= optind;
	argv += optind;

	if (inputs.num != 2) {
		printf(main_help);
		exit(1);
	}

	vac_input_validate(&inputs);

	if (inputs.err_fsize) {
		vac_error("Frame size mismatch. PSNR cannot be computed.");
		exit(1);
	}

	if (inputs.err_frate)
		vac_info("Frame rate mismatch. PSNR may be inaccurate.");

	if (inputs.err_total)
		vac_info("Total number of frames mismatch. PSNR may be inaccurate.");

	process_inputs();
	
	return 0;
}
